@@ -1,3 +1,5 @@
+#ifndef _BUFFER_H
+#define _BUFFER_H
 #include "dact.h"
 
 int bit_buffer_read(int bits);
@@ -14,3 +16,4 @@ void byte_buffer_purge(void);
 #ifndef BYTE_BUFF_SIZE
 #define BYTE_BUFF_SIZE DACT_BLK_SIZE*2
 #endif
+#endif
