@@ -25,10 +25,10 @@ int cipher_sub(const char *inblock, char *outblock, const int blksize, char *key
 			return(cipher_sub_init(mode,key));
 			break;
 		case DACT_MODE_CDEC:
-			return(cipher_sub_encrypt(inblock, outblock, blksize, key));
+			return(cipher_sub_decrypt(inblock, outblock, blksize, key));
 			break;
 		case DACT_MODE_CENC:
-			return(cipher_sub_decrypt(inblock, outblock, blksize, key));
+			return(cipher_sub_encrypt(inblock, outblock, blksize, key));
 			break;
 	}
 	return(0);
@@ -46,68 +46,92 @@ int cipher_sub_init_getkey(const int mode, char *key) {
 	fname=dact_ui_getuserinput("Key file: ",128,0);
 	if (mode==DACT_MODE_CDEC) {
 		fd=open(fname, O_RDONLY);
-		read(fd, key, 256);
+		read(fd, key, 257);
 		close(fd);
-		return(256);
+		return(257);
 	}
 	if (mode==DACT_MODE_CENC) {
 		fd=open(fname, O_WRONLY|O_CREAT|O_TRUNC, 0600);
-		key=generatekey();
-		write(fd, key, 256);
+		memcpy(key,generatekey(),257);
+		write(fd, key, 257);
 		close(fd);
-		return(256);
+		return(257);
 	}
 	return(-1);
 }
 
 int cipher_sub_encrypt(const char *inblk, char *outblk, int blksize, char *key) {
-	memcpy(outblk,cryptdata(key,inblk,blksize),blksize);
+	char *tmpbuf;
+	tmpbuf=cryptdata(key,inblk,blksize);
+	memcpy(outblk,tmpbuf,blksize);
+	free(tmpbuf);
 	return(blksize);
 }
 
 int cipher_sub_decrypt(const char *inblk, char *outblk, int blksize, char *key) {
-	memcpy(outblk,decryptdata(key,inblk,blksize),blksize);
+	char *tmpbuf;
+	tmpbuf=decryptdata(key,inblk,blksize);
+	memcpy(outblk,tmpbuf,blksize);
+	free(tmpbuf);
 	return(blksize);
 }
 
-char *cryptdata(unsigned char key[256], const unsigned char *data, int size) {
-	int i;
-	static char *result;
-        result=malloc(size);
+char *cryptdata(unsigned char key[257], const unsigned char *data, int size) {
+	int i,mod;
+	char *result;
+	static int keyoffset=0;
+	result=malloc(size);
+
+	mod=(int) key[0];
 	for (i=0;i<size;i++) {
-		result[i]=key[(int) data[i]];
+		if (!(i%mod)) {
+			keyoffset=((keyoffset+1)&0xff);
+		}
+		result[i]=key[((((int) data[i])+keyoffset)&0xff)+1];
 	}
 	return(result);
 }
 
-char *decryptdata(unsigned char key[256], const unsigned char *data, int size) {
-	int i;
+char *decryptdata(unsigned char key[257], const unsigned char *data, int size) {
+	int i,mod,x;
 	char reversekey[256];
-	static char *result;
+	char *result;
+	static int keyoffset=0;
+
+	mod=(int) key[0];
 	result=malloc(size);
-	for (i=0;i<256;i++) reversekey[(int) key[i]]=i;
+	for (i=1;i<257;i++) reversekey[(int) key[i]]=(i-1);
 	for (i=0;i<size;i++) {
+		if (!(i%mod)) {
+			keyoffset=((keyoffset+1)&0xff);
+			for (x=0;x<256;x++) reversekey[(int) key[((x+keyoffset)&0xff)+1]]=x;
+		}
 		result[i]=reversekey[(int) data[i]];
 	}
 	return(result);
 }
 
 char *generatekey(void) {
-	static char key[256];
+	static char key[257];
 	char used[256];
 	int i,x;
 #ifdef RANDOM_DEV
 	unsigned char buff[1];
 	int fd;
 	fd=open(RANDOM_DEV, O_RDONLY);
+	read(fd, buff, 1);
+	key[0]=buff[0];
+#else
+	srand(time(NULL)+rand());
+	key[0]=1+(int) (255.0*rand()/(RAND_MAX+1.0));
 #endif
 	for (i=0;i<256;i++) used[i]=0;
-	for (i=0;i<256;i++) {
+	for (i=1;i<257;i++) {
 #ifdef RANDOM_DEV
 		read(fd, buff, 1);
 		x=buff[0];
 #else
-		srand(time(NULL)%rand());
+		srand(time(NULL)+rand());
 		x=1+(int) (255.0*rand()/(RAND_MAX+1.0));
 #endif
 		if (used[x]==0) { key[i]=x; used[x]=1; } else { i--; }
