@@ -11,21 +11,13 @@
 
 int (*(ciphers[CIPHER_COUNT]))()={
 	cipher_chaos,
-#ifdef DEBUG
 	cipher_sub,
-#else
-	NULL,
-#endif
 	NULL
 };
 
 char *ciphers_name[CIPHER_COUNT]={ 		/* These must be lower-cased */
 	"chaos",
-#ifdef DEBUG
 	"subst",
-#else
-	NULL,
-#endif
 	NULL
 };
 
