@@ -34,6 +34,11 @@
 	out_block	- Where to put data after compression.
 	blk_size	- Size of prev_block and curr_block.
 */
+#if defined(AS_MODULE) && defined(USE_MODULES)
+int DC_NUM=10;
+void *DC_ALGO=comp_factor_algo;
+char *DC_NAME="Factor Compression (MOD)";
+#endif
 int comp_factor_algo(int mode, unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size) {
 	switch(mode) {
 		case DACT_MODE_COMPR:
