@@ -15,6 +15,7 @@
 #include "parse.h"
 #include "dact.h"
 #include "crc.h"
+#include "ui.h"
 
 uint32_t atoi2(const char *n) {
 	uint32_t retval=0;
@@ -88,9 +89,13 @@ int parse_url(const char *url, char *scheme, char *username, char *password, cha
 		@@OSVR@@	OS version (2.2.x, 4.2, 5.8, etc)
 		@@OSVS@@	OS version (short) (2.2, 4.2, 5.8, etc)
 		@@ARCH@@	Arch (i386, sparc64, sun4u, sun4m, etc)
-		@@LCVR@@	LibC version
 		@@DIST@@	If OSNM=Linux, distribution of Linux.
 		@@FILE@@	Name of compressed file.
+		@@DTVR@@	Version of DACT (maj.min.rev)
+		@@DTVS@@	Version of DACT (short) (maj.min)
+		@@PASS@@	Prompt for Password
+		@@USER@@	Prompt for Username
+		@@ATSN@@	Put an `@'
 */
 char *parse_url_subst(const char *src, const char *fname) {
 	struct utsname system_info;
@@ -136,6 +141,8 @@ char *parse_url_subst(const char *src, const char *fname) {
 					if (isdigit(system_info.release[x]) || system_info.release[x]=='.') {
 						ret[0]=system_info.release[x];
 						ret++;
+					} else {
+						break;
 					}
 				}
 				break;
@@ -156,10 +163,6 @@ char *parse_url_subst(const char *src, const char *fname) {
 				memcpy(ret,fname,x);
 				ret+=x;
 				break;
-			case 329906: /* LCVR */
-				memcpy(ret, "6.0", 3);
-				ret+=3;
-				break;
 			case 298628: /* DIST-done */
 				if (!strcmp("linux",system_info.sysname)) {
 					smbuf=parse_url_subst_dist();
@@ -170,6 +173,42 @@ char *parse_url_subst(const char *src, const char *fname) {
 					ret+=x;
 				}
 				break;
+			case 301490: /* DTVR-done */
+				smbuf=malloc(128);
+				sprintf(smbuf, "%i.%i.%i", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION);
+				x=strlen(smbuf);
+				memcpy(ret, smbuf, x);
+				ret+=x;
+				free(smbuf);
+				break;
+			case 301491: /* DTVS-done */
+				smbuf=malloc(128);
+				sprintf(smbuf, "%i.%i", DACT_VER_MAJOR, DACT_VER_MINOR);
+				x=strlen(smbuf);
+				memcpy(ret, smbuf, x);
+				ret+=x;
+				free(smbuf);
+				break;
+			case 345731: /* PASS-done */
+				smbuf=dact_ui_getuserinput("Enter password: ", 128, 1);
+				x=strlen(smbuf);
+				if (x>127) break;
+				memcpy(ret, smbuf, x);
+				ret+=x;
+				free(smbuf);
+				break;
+			case 370594: /* USER-done */
+				smbuf=dact_ui_getuserinput("Enter username: ", 128, 0);
+				x=strlen(smbuf);
+				if (x>127) break;
+				memcpy(ret, smbuf, x);
+				ret+=x;
+				free(smbuf);
+				break;
+			case 289150: /* ATSN-done */
+				ret[0]='@';
+				ret+=1;
+				break;
 #ifdef DEBUG
 			default:
 				PRINT_LINE; fprintf(stderr, "Unknown cmd (%i) [src=%s].\n",cmd,src);
@@ -257,8 +296,9 @@ ssize_t read_f(int fd, void *buf, size_t count) {
 	ssize_t i=0,x;
 
 	while (i!=count) {
-		x=read(fd,buf+i,count-i);
+		x=read(fd,(void *) ((buf)+i),count-i);
 		if (x==0) break;
+		if (x==-1) return(-1);
 		i+=x;
 	}
 
